// -*- coding: utf-8 -*-
// SM Tuning.
//
// Funkcje i stałe związane z obsługą zbiorów grafik

// Stałe pomocnicze, tylko do poniższej inicjalizacji

///////////////////////////////////////////////////////////////////////
// Reprezentacja samej informacji o tle. Wyłącznie nazwa wewnętrzna
// i zewnętrzna.
///////////////////////////////////////////////////////////////////////

// new SmtImage('brown', 'Brown')
function SmtBackground(name, label) {
    this.name = name;
    this.label = label;
}

// Podmienia w url-u fragmenty dotyczące tła
SmtBackground.prototype.patchUrl = function(url) {
    return url.replace('%bgr', this.name);
}

///////////////////////////////////////////////////////////////////////
// Opis zbioru obrazków
///////////////////////////////////////////////////////////////////////

// new SmtFigureSet(
//          'fantasy', 'Fantasy',
//          'chrome://smtuning/content/pieces/%name__%bgr/%size/%piece.gif',
//          [ 32, 40, 48, 64 ],
//          /* tu np. 'makruk' */
//          /* tu ewentualnie '.jpg' */
//     );
//
function SmtFigureSet(name, label, url, sizes, used_for, ext) {
    this.name = name;
    this.label = label;
    this.url = url.replace('%name', name);
    this.sizes = sizes;
    this.used_for = used_for || "";
    this.ext = ext || ".gif";
}

SmtFigureSet.prototype.hasSize = function(size) {
    for (var idx = 0, sz = this.sizes.length;
         idx < sz;
         ++ idx) {
        if (this.sizes[idx] == size) {
            return true;
        }
    }
    return false;
}

SmtFigureSet.prototype.fileExt = function() {
    return this.ext;
}

SmtFigureSet.prototype.applicable = function(variant) {
    if (variant.match(/(shatranj|makruk|blindfold)/)) {
        return this.used_for == variant;
    }
    else {
        return ! this.used_for;
    }
}


// Zwraca większy (następny) rozmiar od podanego albo null gdy nie ma
SmtFigureSet.prototype.nextSizeUp = function(size) {
    for (var idx = 0, sz = this.sizes.length - 1;
         idx < sz;
         ++ idx) {
        if (this.sizes[idx] == size) {
            return this.sizes[idx+1];
        }
    }
    return null;
}
// Zwraca mniejszy (następny) rozmiar od podanego albo null gdy nie ma
SmtFigureSet.prototype.nextSizeDown = function(size) {
    for (var idx = 1, sz = this.sizes.length;
         idx < sz;
         ++ idx) {
        if (this.sizes[idx] == size) {
            return this.sizes[idx-1];
        }
    }
    return null;
}

SmtFigureSet.prototype.pieceUrl = function(piece,size) {
    var u = this.url;
    return u.replace('%size', size.toString())
            .replace('%piece', piece);
}

///////////////////////////////////////////////////////////////////////
// Reprezentacja właściwego piece-set-a (konkretne figurki
// na konkretnym tle)
///////////////////////////////////////////////////////////////////////
    
// background może być null-em, to oznacza że tła są zaszyte w figureSet
function SmtPieceSet(figureSet, background) {
    // smtext.dump("SmtPieceSet(" + figureSet + ", " + background + ")");
    this.fig = figureSet;
    this.bgr = background;
}
                         
SmtPieceSet.prototype.hasSize = function(size) {
    return this.fig.hasSize(size);
}

SmtPieceSet.prototype.applicable = function(variant) {
    return this.fig.applicable(variant);
}

// set.pieceUrl('wkw', 40);
SmtPieceSet.prototype.pieceUrl = function(piece, size) {
    var u = this.fig.pieceUrl(piece, size);
    if (this.bgr) {
        u = this.bgr.patchUrl(u);
    }
    return u;
}

SmtPieceSet.prototype.piecesPath = function(size) {
    var u = this.pieceUrl('XXXX', size);
    return u.replace(/XXXX\.(gif|png|jpg)/, '');
}

// Zwraca większy (następny) rozmiar od podanego albo null gdy nie ma
SmtPieceSet.prototype.nextSizeUp = function(size) {
    return this.fig.nextSizeUp(size);
}
// Zwraca mniejszy (następny) rozmiar od podanego albo null gdy nie ma
SmtPieceSet.prototype.nextSizeDown = function(size) {
    return this.fig.nextSizeDown(size);
}

SmtPieceSet.prototype.name = function() {
    if (this.bgr) {
        return this.fig.name + '__' + this.bgr.name;
    }
    else {
        return this.fig.name;
    }
}

SmtPieceSet.prototype.label = function() {
    if (this.bgr) {
        return this.fig.label + '/' + this.bgr.label;
    }
    else {
        return this.fig.label;
    }
}

SmtPieceSet.prototype.fileExt = function() {
    return this.fig.fileExt();
}

///////////////////////////////////////////////////////////////////////
// Obiekt zarządczy - pakiet figurek i różne funkcje wyszukiwawcze
///////////////////////////////////////////////////////////////////////

function SmtAllPieces() {
    this.mixes = [];
    this.plains = [];
    this.pieceAliases = {};
    this.figureAliases = {};
}
// Dodaje wszystkie kombinacje
SmtAllPieces.prototype.addMixes = function(figuresList, backgroundsList) {
    this.mixes[ this.mixes.length ] = {
        figures: figuresList,
        backgrounds: backgroundsList
    };
}
// Dodaje figurki bez wariantów tła
SmtAllPieces.prototype.addPlains = function(figuresList) {
    for(var i=0, img; img=figuresList[i]; i++) {
        this.plains[ this.plains.length ] = img;
    }
}

// Dodaje alias dla samych figurek
SmtAllPieces.prototype.aliasFigure = function(alias, trueName) {
    this.figureAliases[alias] = trueName;
}

// Dodaje alias dla całości
SmtAllPieces.prototype.aliasPiece = function(alias, figureName, backgroundName) {
    this.pieceAliases[alias] = [ figureName, backgroundName ];
}

// Wyszukanie obiektu SmtPieceSet na podstawie technicznej nazwy
SmtAllPieces.prototype.pieceByName = function(name) {
    // smtext.dump("Searching for '" + name + "'");
    var l = name.split("__");
    var figName = null;
    var bgrName = null;
    if (l.length == 2) {
        figName = l[0];
        bgrName = l[1];
    }
    else {
        // szukamy name, może być albo aliasem albo singularem
        var x = this.pieceAliases[name];
        if (x) {
            figName = x[0];
            bgrName = x[1];
            // smtext.dump("Found as piece alias for " + figName + ", " + bgrName);
        }
        else {
            for (var i = 0, fig; fig = this.plains[i]; i++) {
                if (fig.name == name) {
                    // smtext.dump("Resolved as old style piece " + name)
                    return new SmtPieceSet(fig, null);
                }
            }
        }
    }

    var x = this.figureAliases[figName];
    if (x) {
        // smtext.dump("Found as figure alias for " + x)
        figName = x;
    }

    // Mamy figName i bgrName, musimy poszukać
    for (var i = 0, mix; mix=this.mixes[i]; i++) {
        var fig, bgr;
        for (var j = 0, f; f=mix.figures[j]; j++) {
            if (f.name == figName) {
                // smtext.dump("Found new style piece " + figName);
                fig = f;
                break;
            }
        }
        for (var k = 0, b; b=mix.backgrounds[k]; k++) {
            if (b.name == bgrName) {
                // smtext.dump("Found new style bgr " + bgrName);
                bgr = b;
                break;
            }
        }
        if (fig && bgr) {
            return new SmtPieceSet(fig, bgr);
        }
    }

    // Awaryjnie
    // smtext.dump("Emergency piece lookup");
    return this.firstMatchingPiece("chess");
}

// Wyszukanie pierwszego pasującego do podanego wariantu
SmtAllPieces.prototype.firstMatchingPiece = function(variant) {
    for (var i = 0, mix; mix=this.mixes[i]; i++) {
        for (var j = 0, f; f=mix.figures[j]; j++) {
            if (f.applicable(variant)) {
                return new SmtPieceSet(f, mix.backgrounds[0]);
            }
        }
    }
    return null;
},

// same_applicable(variant1, variant2)
// Czy zbiór pasujących jest ten sam
SmtAllPieces.prototype.same_applicable = function(variant1, variant2) {
    if (variant1 == variant2) {
        return true;
    }
    if (! variant1.match(/(shatranj|makruk|blindfold)/)
        && ! variant2.match(/(shatranj|makruk|blindfold)/) ) {
        return true;
    }
    // Różne i choć jeden jest shatranjem lub makrukiem
    return false;
}

// Zwraca listę obiektów SmtPiece obsługujących podany rozmiar
// i pasujących do podanego wariantu
SmtAllPieces.prototype.allSuitable = function(variant, size) {
    var tab = [];

    for (var i = 0, mix; mix=this.mixes[i]; i++) {
        for (var j = 0, fig; fig=mix.figures[j]; j++) {
            if (fig.hasSize(size) && fig.applicable(variant)) {
                for (var k = 0, bgr; bgr=mix.backgrounds[k]; k++) {
                    tab[ tab.length ] = new SmtPieceSet(fig, bgr);
                }
            }
        }
    }
    for (var i = 0, fig; fig = this.plains[i]; i++) {
        if (fig.hasSize(size) && fig.applicable(variant)) {
            tab[ tab.length ] = new SmtPieceSet(fig, null);
        }
    }

    return tab;
}

///////////////////////////////////////////////////////////////////////

var smtpieces = (function() {
        var x = new SmtAllPieces();
        //var SmtLocalPiecesUrl = 'chrome://smtuning/content/pieces/%name/%size/%piece.gif';
        var wikiUrl = 'http://wiki.schemingmind.com/pieces/%name__%bgr/%size/%piece.gif';
        var smUrl = 'http://www.schemingmind.com/images/sets/%name/%piece.gif';
        var smJpgUrl = 'http://www.schemingmind.com/images/sets/%name/%piece.jpg';

        x.addMixes(
                   [
                    new SmtFigureSet('igor_k_shadow', 'Igor-K',
                                     wikiUrl, [32,40,48,64]),
                    new SmtFigureSet('fantasy', 'Fantasy',
                                     wikiUrl, [32,40,48,64]),
                    new SmtFigureSet('makruk', 'Makruk',
                                     wikiUrl, [32,40,48,64],
                                     'makruk'),
                    new SmtFigureSet('shatranj_tamerlane', 'Shatranj Tamerlane',
                                     wikiUrl, [32,40,48,64],
                                     'shatranj'),
                    new SmtFigureSet('spatial', 'Spatial',
                                     wikiUrl, [32,40,48,64]),
                    new SmtFigureSet('skulls', 'Skulls',
                                     wikiUrl, [32,40,48,64]),
                    new SmtFigureSet('eyes', 'Eyes',
                                     wikiUrl, [32,40,48,64]),
                    ],
                   [
                    new SmtBackground('paper_brown', 'Brown Paper'),
                    new SmtBackground('paper_khaki', 'Khaki Paper'),
                    new SmtBackground('paper_violet', 'Violet Paper'),
                    new SmtBackground('solid_darkbrown', 'Dark Brown'),
                    new SmtBackground('solid_darkgreen', 'Dark Green'),
                    new SmtBackground('solid_greenbufflike', 'GreenBuff'),
                    new SmtBackground('solid_grey', 'Grey'),
                    new SmtBackground('solid_lightbrown', 'Light Brown'),
                    new SmtBackground('solid_mongeblue', 'Blue'),
                    new SmtBackground('solid_redlike', 'Red'),
                    new SmtBackground('solid_yellow', 'Yellow'),
                    new SmtBackground('texture_stone', 'Stone'),
                    new SmtBackground('texture_walnut', 'Walnut'),
                    new SmtBackground('texture_wood_pine', 'Pine Wood'),
                    ]);

        x.addPlains([
                     new SmtFigureSet('default', 'Default',
                                      smUrl, [32]),
                     new SmtFigureSet('fritzwood', 'Fritz Wood',
                                      smUrl, [32]),
                     new SmtFigureSet('greenbuff', 'Green Buff',
                                      smUrl, [32]),
                     new SmtFigureSet('cookie', 'Cookie',
                                      smUrl, [32]),
                     new SmtFigureSet('harlequin', 'Harlequin',
                                      smUrl, [32]),
                     new SmtFigureSet('jackson', 'Jackson',
                                      smUrl, [32]),
                     new SmtFigureSet('leipzig', 'Leipzig',
                                      smUrl, [32]),
                     new SmtFigureSet('strategy', 'Strategy',
                                      smUrl, [32]),
                     new SmtFigureSet('crown_jewels', 'Crown Jewels',
                                      smUrl, [32]),
                     new SmtFigureSet('shatranj', 'Shatranj',
                                      smUrl, [32],
                                      'shatranj'),
                     new SmtFigureSet('shatranj_nishapur', 'Shatranj Nishapur',
                                      smUrl, [32],
                                      'shatranj'),
                     new SmtFigureSet('oldbook', 'Old Book',
                                      smJpgUrl, [32], '', '.jpg'),
                     new SmtFigureSet('blindfold', 'Blindfold',
                                      smUrl, [32],
                                      'blindfold'),
                     ]);

        // Aliasy - obsługa rzeczy które były w starszych wersjach

        x.aliasFigure('igor_k', 'igor_k_shadow');

        x.aliasPiece('fantasy', 'fantasy', 'solid_mongeblue');
        x.aliasPiece('fantasy_green', 'fantasy', 'solid_greenbufflike');
        x.aliasPiece('fantasy_brown', 'fantasy', 'solid_lightbrown');
        x.aliasPiece('igor_k', 'igor_k_shadow', 'solid_darkbrown');
        x.aliasPiece('igor_k_green', 'igor_k_shadow', 'solid_greenbufflike');
        x.aliasPiece('igor_k_light', 'igor_k_shadow', 'solid_lightbrown');
        x.aliasPiece('igor_k_grey', 'igor_k_shadow', 'solid_grey');
        x.aliasPiece('igor_k_paper', 'igor_k_shadow', 'paper_brown');
        x.aliasPiece('igor_k_wood', 'igor_k_shadow', 'wood_pine');
        x.aliasPiece('igor_k_walnut', 'igor_k_shadow', 'walnut');
        x.aliasPiece('spatial', 'spatial', 'solid_mongeblue');
        x.aliasPiece('skulls', 'skulls', 'solid_mongeblue');
        x.aliasPiece('eyes', 'eyes', 'solid_mongeblue');
        x.aliasPiece('shatranj_tamerlane', 'shatranj_tamerlane', 'solid_grey');
        x.aliasPiece('shatranj_tamerlane_redlike', 'shatranj_tamerlane', 'solid_redlike');
        x.aliasPiece('makruk', 'makruk', 'solid_darkgreen');
        x.aliasPiece('makruk_grey', 'makruk', 'solid_grey');

        return x;
    })();

